#include <ESP8266WiFi.h>
#include <MQTT.h>

const char ssid[] = "StudentResidences";
const char pass[] = "";
WiFiClient net;
MQTTClient client;

// Add variables for counting and time tracking
int lastValueL = -1, lastValueR = -1;
unsigned long lastUpdateTime = 0;
const long delayTime = 5000;  // 5 second delay

void connect() {
  Serial.print("checking wifi...");
  WiFi.begin(ssid, pass);
  while (WiFi.status() != WL_CONNECTED) delay(1000);
  client.begin("test.mosquitto.org", net);
  }
    
  Serial.println("Connected to WiFi!"); //Debugging

  while (!client.connect("MonkeysensorL")) {
    Serial.print(".");
    delay(1000);
  }
  Serial.print("\nconnecting...");
  // Print the client ID ("MonkeysensorL") to the serial monitor
  Serial.print("Connecting to MQTT with client ID: ");
  Serial.println("MonkeysensorL");
  
  // Change "mySessionID" to your own identifier (name, project etc)
  while (!client.connect("MonkeysensorL", "", "")) {
    Serial.print(".");
    delay(1000);
  }
  
  Serial.println("Connected to MQTT Broker!"); //Debugging

  //subscribing to the mqtt topics
  client.subscribe("MonkeysensorL"); 
  client.subscribe("MonkeysensorR");
}

void messageReceived(String &topic, String &payload) {
  
  Serial.print("Received message on topic: "); // prints this text to the serial monitor for debugging
  Serial.println(topic); // Prints the value of the topic to the next line in the serial monitor for debugging
  Serial.print("Payload: "); // Prints this text to the serial monitor for debugging
  Serial.println(payload); // Prints the value recieved from mqtt to the next line in the serial monitor for debugging 
  
  // Connects to the topic on the MQTT broker and returns the most recent data in that topic.
  // Data is held in the variable "payload"
  int sensorValue = payload.toInt(); // Convert the payload to an integer
  if (millis() - lastUpdateTime < delayTime) return; // Ignores the following code if the elapsed time since the last update is less than the 5000 millisecond delay
  
  if (topic == "MonkeysensorL" && sensorValue != lastValueL) { //Makes sure that the program only runs when something is detected by the sensor, by checking whether both MonkeysensorL and the intager from the mqtt broker are different from lastValueL
    Serial.println("+1"); // Adds 1 to the internal count
    lastValueL = sensorValue; // Updates the sensor value to compare new sensor readings with the previous one
    lastUpdateTime = millis(); // Update the last update time for MonkeysensorL
  }
  else if (topic == "MonkeysensorR" && sensorValue != lastValueR) {
    Serial.println("-1"); // Subtracts 1 to the internal count
    lastValueR = sensorValue;
    lastUpdateTime = millis(); // Update the last update time for MonkeysensorR
  }
}

void setup() {
  Serial.begin(115200);
  WiFi.begin(ssid, pass);
  client.begin("test.mosquitto.org", net);
  client.onMessage(messageReceived);
  connect();
}

void loop() {
  if (!client.connected()) {
    connect();
  }
  client.loop();
  delay(10); // <- fixes some issues with WiFi stability
}

  if (!client.connected()) connect();
}
